from datetime import timedelta as delta
from datetime import datetime as datetime
from glob import glob
import numpy as np
import xarray as xr
import scipy.stats as stats

from parcels import FieldSet, Field, NestedField, VectorField, ParticleFile, ParticleSet, JITParticle, Variable
from parcels import ErrorCode
from northsea_mp_kernels import *


ddir = 'E:/cmems_data/'
def run_particles(wstokes, unbeaching, years, model='cmems'):
    diffusivity = 1
    unbeachfile = 'E:/Krakatau_parcels/parcels_dataset/cmems_NWS_rean_006_009_unbeaching_vel.nc'
    
    cmemsfiles = sorted(glob(ddir+'SMOC_201?????_R201?????.nc'))
    cmemsvariables = {'U': 'uo', 'V': 'vo'}
    cmemsdimensions = {'lon': 'longitude', 'lat': 'latitude', 'time': 'time'}

    #currents
    fset_currents = FieldSet.from_netcdf(cmemsfiles, cmemsvariables, cmemsdimensions, allow_time_extrapolation=True)
    
    #dimensions
    dimensions = {'lon': 'longitude', 'lat': 'latitude'}
    meshSize = Field.from_netcdf(unbeachfile, ('meshSize', 'unBeachU'), dimensions, interp_method='nearest')

    #tides
    tidesfiles = sorted(glob(ddir+'SMOC_201?????_R201?????.nc'))
    tidesdimensions = {'lat': 'latitude', 'lon': 'longitude', 'time': 'time'}
    tidesvariables = {'U': 'utide', 'V': 'vtide'}
    fset_tides = FieldSet.from_netcdf(tidesfiles, tidesvariables, tidesdimensions, allow_time_extrapolation=True)
    
    #tides+currents
    fieldset = FieldSet(U=fset_currents.U+fset_tides.U, V=fset_currents.V+fset_tides.V)

    #winds
    # windfiles = 
    
    if wstokes:
        stokesfiles = sorted(glob(ddir+'SMOC_201?????_R201?????.nc'))
        
        #stokes
        dimensionsU = {'lon': 'longitude', 'lat': 'latitude', 'time': 'time'}
        dimensionsV = {'lon': 'longitude', 'lat': 'latitude', 'time': 'time'}
        Uuss = Field.from_netcdf(stokesfiles, ('Uuss', 'vsdx'), dimensionsU, fieldtype='U', allow_time_extrapolation=True)
        Vuss = Field.from_netcdf(stokesfiles, ('Vuss', 'vsdy'), dimensionsV, fieldtype='V', allow_time_extrapolation=True, grid=Uuss.grid, dataFiles=Uuss.dataFiles)

        fieldset.add_field(Uuss)
        fieldset.add_field(Vuss)
        fieldset.Uuss.vmax = 5
        fieldset.Vuss.vmax = 5
        uv_uss = VectorField('UVuss', fieldset.Uuss, fieldset.Vuss)
        fieldset.add_vector_field(uv_uss)

    if unbeaching:

        #unbeaching
        dimensionsU = {'lon': 'longitude', 'lat': 'latitude'}
        dimensionsV = {'lon': 'longitude', 'lat': 'latitude'}
        Ucmems_unbeach = Field.from_netcdf(unbeachfile, ('Ucmems_unbeach', 'unBeachU'), dimensionsU, fieldtype='U')
        Vcmems_unbeach = Field.from_netcdf(unbeachfile, ('Vcmems_unbeach', 'unBeachV'), dimensionsV, fieldtype='V')

        fieldset.add_field(Ucmems_unbeach)
        fieldset.add_field(Vcmems_unbeach)
        UVunbeach = VectorField('UVunbeach', fieldset.Ucmems_unbeach, fieldset.Vcmems_unbeach)
        fieldset.add_vector_field(UVunbeach)
    
    fieldset.add_field(meshSize)
    fieldset.add_field(Field('Kh_zonal', data=diffusivity*np.ones(meshSize.data.shape), grid=meshSize.grid))
    fieldset.add_field(Field('Kh_meridional', data=diffusivity*np.ones(meshSize.data.shape), grid=meshSize.grid))

    ####################################################################################################################

    fieldset.U.vmax = 5
    fieldset.V.vmax = 5
    fieldset.cmems = True

    #coordinates
    startlats = [-6.04,-6.04,-6.04,-6.05,-6.06,-5,-7]
    startlons = [105.24,105.25,105.26,105.25,105.25,105,113]
    nparticles = [50,50,50,50,50,0,0]
   
    nppersite = 1.
    np.random.seed(1234)
    allstartlon = []
    allstartlat = []
    allsitenumbers = []
    for s in range(len(startlats)):
        for i in range(int(nparticles[s]*nppersite)):
            allstartlon.append(startlons[s] + np.random.uniform(-10, 10) / 1.852 / 60. / np.cos(startlats[s] * np.pi / 180))
            allstartlat.append(startlats[s] + np.random.uniform(-10, 10) / 1.852 / 60.)
            allsitenumbers.append(s+1)

    for y in years:
        ws = '_wstokes' if wstokes else ''
        ub = '_unbeach' if unbeaching else '_beach'
        # fname = "9_month_test_%s_%d%s%s.nc" % (model, y, ws, ub)
        fname = "krakrun.nc"

        allstarttime = []
        for i in range(len(allstartlon)):
            ddays = int(stats.truncnorm(-30./20, 30./20, loc=0, scale=20).rvs(1))
            allstarttime.append(datetime(2018, 12, 22, 0, 0, 0))
            #12-22
            

        class PumiceParticle(JITParticle):
            beached = Variable('beached', dtype=np.int32, initial=0.)
            unbeachCount = Variable('unbeachCount', dtype=np.int32, initial=0.)
            sitenumber = Variable('sitenumber', dtype=np.int32, to_write='once')

        pset = ParticleSet(fieldset=fieldset, pclass=PumiceParticle, lon=allstartlon, lat=allstartlat,
                           time=allstarttime, sitenumber=allsitenumbers)
        outfile = pset.ParticleFile(name=fname, outputdt=delta(days=1))

        # kernels = pset.Kernel(AdvectionRK4) + SampleSpeed
        kernel = pset.Kernel(AdvectionRK4)
        #BeachTesting = 
        kernel += pset.Kernel(BeachTesting_2D) + pset.Kernel(UnBeaching)
        if wstokes:
            kernel += pset.Kernel(StokesDrag) + pset.Kernel(BeachTesting_2D)
        # if diffusion > 0:
        kernel += pset.Kernel(BrownianMotion2D) + pset.Kernel(BeachTesting_2D)


        pset.execute(kernel, dt=delta(hours=1), endtime=datetime(2019, 6, 30, 0, 0, 0), output_file=outfile)
        #outfile.close()


for yr in np.arange(2019, 2020):
    for unbeaching in [True]:
        for wstokes in [True]:
            run_particles(wstokes, unbeaching, [yr])

